#  ***** BEGIN LICENSE BLOCK *****  
#   
#  Source last modified: $Id: umake_win_vc8_makefile.py,v 1.4 2008/05/01 20:47:01 jfinnecy Exp $ 
#   
#  Copyright Notices: 
#   
#  Portions Copyright (c) 1995-2006 RealNetworks, Inc. All Rights Reserved. 
#   
#  Patent Notices: This file may contain technology protected by one or  
#  more of the patents listed at www.helixcommunity.org 
#   
#  1.   The contents of this file, and the files included with this file, 
#  are protected by copyright controlled by RealNetworks and its  
#  licensors, and made available by RealNetworks subject to the current  
#  version of the RealNetworks Public Source License (the "RPSL")  
#  available at  * http://www.helixcommunity.org/content/rpsl unless  
#  you have licensed the file under the current version of the  
#  RealNetworks Community Source License (the "RCSL") available at 
#  http://www.helixcommunity.org/content/rcsl, in which case the RCSL 
#  will apply.  You may also obtain the license terms directly from 
#  RealNetworks.  You may not use this file except in compliance with 
#  the RPSL or, if you have a valid RCSL with RealNetworks applicable 
#  to this file, the RCSL.  Please see the applicable RPSL or RCSL for 
#  the rights, obligations and limitations governing use of the 
#  contents of the file. 
#   
#  2.  Alternatively, the contents of this file may be used under the 
#  terms of the GNU General Public License Version 2 (the 
#  "GPL") in which case the provisions of the GPL are applicable 
#  instead of those above.  Please note that RealNetworks and its  
#  licensors disclaim any implied patent license under the GPL.   
#  If you wish to allow use of your version of this file only under  
#  the terms of the GPL, and not to allow others 
#  to use your version of this file under the terms of either the RPSL 
#  or RCSL, indicate your decision by deleting Paragraph 1 above 
#  and replace them with the notice and other provisions required by 
#  the GPL. If you do not delete Paragraph 1 above, a recipient may 
#  use your version of this file under the terms of any one of the 
#  RPSL, the RCSL or the GPL. 
#   
#  This file is part of the Helix DNA Technology.  RealNetworks is the 
#  developer of the Original Code and owns the copyrights in the 
#  portions it created.   Copying, including reproducing, storing,  
#  adapting or translating, any or all of this material other than  
#  pursuant to the license terms referred to above requires the prior  
#  written consent of RealNetworks and its licensors 
#   
#  This file, and the files included with this file, is distributed 
#  and made available by RealNetworks on an 'AS IS' basis, WITHOUT  
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND REALNETWORKS  
#  AND ITS LICENSORS HEREBY DISCLAIM  ALL SUCH WARRANTIES, INCLUDING  
#  WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS  
#  FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. 
#   
#  Technology Compatibility Kit Test Suite(s) Location:  
#     http://www.helixcommunity.org/content/tck 
#   
# Contributor(s):
#   This file was contributed by Christian Buchner
# 
# ***** END LICENSE BLOCK *****
# 
# 
""" Makefile generator which generates nmake files and VC8 SLN/VCPROJ files """

import os
import string

import log
log.debug("Imported: $Id: umake_win_vc8_makefile.py,v 1.4 2008/05/01 20:47:01 jfinnecy Exp $")
import umake

import umake_win_vc7_makefile
# Convenience mapping to xml class.
xml = umake_win_vc7_makefile.xml

import umake_win_makefile
import umake_makefile

class COptionEHHandler(umake_win_vc7_makefile.CompilerOptionHandler):
    def handle(self):
        if self.arg[0] == "s":
            self.config.toolopts["ExceptionHandling"]=self.config.OneorZero(self.arg[1:])
            return
        self.notHandled()
        
class COptionV8GHandler(umake_win_vc7_makefile.CompilerOptionHandler):
    def handle(self):
        # We only need to override the GX behavior.
        if self.arg[0] == 'X':
            self.config.toolopts["ExceptionHandling"]=self.config.OneorZero(arg[1:])
        else:
            # Any other brands of /G we can reuse the GHandler from VC7.
            umake_win_vc7_makefile.COptionGHandler(self.opt, self.arg, self.config).handle()            
    
class COptionV8MHandler(umake_win_vc7_makefile.CompilerOptionHandler):
    def handle(self):
        # We want to ignore the /ML or /MLD. Otherwise, we can handle M per
        # the VC7 handling rules.
        if self.arg == "L" or self.arg == "LD":
            handler = umake_win_vc7_makefile.COptionNotHandled
        else:
            handler = umake_win_vc7_makefile.COptionMHandler

        handler(self.opt, self.arg, self.config).handle()
        
class VC8Config(umake_win_vc7_makefile.VC7Config):
    """Class representing one profile in a MSDEV project"""

    def __init__(self,
                 type,
                 variables,
                 target_type,
                 post_build = []):
                     
        umake_win_vc7_makefile.VC7Config.__init__(self,type,variables,target_type,post_build)
        
        # Set up VC8 specific compiler option handlers.
        self.compilerOptions['EH'] = COptionEHHandler
        self.compilerOptions['Zc'] = umake_win_vc7_makefile.COptionZmHandler
        self.compilerOptions['G']  = COptionV8GHandler
           
        
    # New method to return a 1 or 0 based on the arg being "-".
    def OneorZero(self, arg):
        if arg == "-":
            return "0"
        return "1"
        
    def setCompilerToolOpts(self):
        # vs vc7: Different set of allowable options. Adds ZC and EH.
        self.cc_toolopts=self.parse_options("VCCLCompilerTool",
                                            "ALL_CC_FLAGS",
                                            self.CompilerOption,
                                            "D=I=Zm=Zp=Zc=FI=W=EH=O=M=G=Y=F=wd=Fd=",
                                            defopts={"Optimization":"0" })
        # vs vc7: add so that wchar_t maps to UINT16                                            
        self.cc_toolopts["TreatWChar_tAsBuiltInType"]="false"

    def setToolXML(self , project):
        # vs vc7: complete rewrite of conftype != 0 branch.
        if self.conftype != 0:
            self.toolXML = [
                xml("Tool",
                    { "Name":"VCPreBuildEventTool", }),
                xml("Tool",
                    { "Name":"VCCustomBuildTool", }),
                xml("Tool",
                    { "Name":"VCXMLDataGeneratorTool", }),
                xml("Tool",
                    { "Name":"VCWebServiceProxyGeneratorTool", }),
                xml("Tool", self.midl_toolopts ),
                xml("Tool", self.cc_toolopts),
                xml("Tool",
                    { "Name":"VCManagedResourceCompilerTool", }),
                xml("Tool", self.rc_toolopts),
                xml("Tool",
                    { "Name":"VCPreLinkEventTool", }),
                ]

            if self.conftype == 4:
                self.toolXML.append(
                    xml("Tool",{ "Name":"VCLibrarianTool",
                                 "OutputFile":self.linker_toolopts["OutputFile"]}))
            else:
                self.toolXML.append(xml("Tool", self.linker_toolopts))

            # vs vc7: The following are all additions.
            self.toolXML.append(xml("Tool",
                    { "Name":"VCALinkTool", }))
            self.toolXML.append(xml("Tool",
                    { "Name":"VCManifestTool", }))
            self.toolXML.append(xml("Tool",
                    { "Name":"VCXDCMakeTool", }))
            self.toolXML.append(xml("Tool",
                    { "Name":"VCBscMakeTool", }))
            self.toolXML.append(xml("Tool",
                    { "Name":"VCFxCopTool", }))
            self.toolXML.append(xml("Tool",
                    { "Name":"VCAppVerifierTool", }))
            self.toolXML.append(xml("Tool",
                    { "Name":"VCWebDeploymentTool", }))
            self.toolXML.append(xml("Tool",
                    { "Name":"VCPostBuildEventTool",
                      "Description":"Copying...",
                      "CommandLine":string.join(self.post_build_cmds,"\n")}))
                
        else:
            self.toolXML= [
                xml("Tool",
                    { "Name":"VCNMakeTool",
                      "BuildCommandLine":
                      "nmake.exe /NOLOGO /f %s all copy" %
                      project.makefile_name,
                      "ReBuildCommandLine":
                      "nmake.exe /NOLOGO /f %s /a all copy" %
                      project.makefile_name,
                      "CleanCommandLine":
                      "nmake.exe /NOLOGO /f %s clean" %
                      project.makefile_name,
                      "Output":self.variables["OUTFILE"] }),
                ]



class VC8Project(umake_win_vc7_makefile.VC7Project):
    """Class representing and generating one MSDEV project"""

    def __init__(self, name, makefile_name, target_type, variables):
        umake_win_vc7_makefile.VC7Project.__init__(self, name, makefile_name, target_type, variables)
        self.projVersion = "8.00"
        
    def getXML(self):
        # vs VC7: adds ToolFiles and References
        return xml("VisualStudioProject",
                   { "ProjectType":"Visual C++",
                     "Version":"%s" % self.projVersion,
                     "Name":self.name,
                     "ProjectGUID":self.guid,
                     "Keyword":"MakeFileProj", },
                   [ xml("Platforms", None,
                         [ xml("Platform",{ "Name":"Win32" } ) ] ),
                     xml("ToolFiles",{},[]),
                     xml("Configurations", {}, self.config_xml),
                     xml("References",{},[]),
                     xml("Files", {},
                         [ xml("Filter",
                               { "Name":"Source Files",
                                 "Filter":"cpp;c;cxx;def;odl;idl;hpj;bat;asm"
                                 },
                               self.file_xml),
                           xml("Filter",
                               { "Name":"Header Files",
                                 "Filter":"h;hpp;hxx;hm;inl;inc",
                                 },
                               self.header_xml),
                           xml("Filter",
                               { "Name":"Resource Files",
                                 "Filter":"rc;ico;cur;bmp;dlg;rc2;rct;bin;rgs;gif;jpg;jpeg;jpe",
                                 },
                               self.resource_xml)
                           ]),
                     xml("Globals",{},[]),
                     ])        

class VC8Workspace(umake_win_vc7_makefile.VC7Workspace):
    """Class representing and generating one MSDEV solution"""

    def __init__(self):
        umake_win_vc7_makefile.VC7Workspace.__init__(self)
        self.version = [
           'Microsoft Visual Studio Solution File, Format Version 9.00',
           '# Visual Studio 2005'
        ]
        self.buildString = "Build|Win32"
        self.projectString = "ProjectConfigurationPlatforms"
              
    def appendTrailingSections(self):
        """appendTrailingSections()
        
        Appends trailing sections to the self.text attribute.
        """
        self.text.append('\tGlobalSection(SolutionProperties) = preSolution')
        self.text.append('\t\tHideSolutionNode = FALSE')
        self.text.append('\tEndGlobalSection')
        
        
class VC8MakefileGenerator( umake_makefile.makefile_generator ):
    """Class for VC8 specific makefile generation overrides. Be careful in
    that there is currently no umake_win_makefile generator, but that class
    (if it is ever created) should sit between umake_makefile and
    umake_win_vc8_makefile in the inheritance tree.
    
    This class was created to enable manifest embedding per the recommended
    MSDN approach:
    http://msdn2.microsoft.com/en-us/library/ms235591(VS.80).aspx"""

    def __init__( self , platform , project ):
        # Add the before and after includes...
        project.pre_target_buff.extend( '!include $(BUILD_ROOT)\umake\makefile.inc\n' )
        project.post_target_buff.extend( '!include $(BUILD_ROOT)\umake\makefile.targ.inc\n' )

        # Then run the base class init().
        umake_makefile.makefile_generator.__init__( self , platform , project )

        
    def LinkTarget( self ):
        # Run the base class LinkTarget()...
        umake_makefile.makefile_generator.LinkTarget( self )
        
        # Then add the manifest embedding calls in the link target.
        if self.project.getTargetType() == 'exe':
            self.writeln( '\t$(_VC_MANIFEST_EMBED_EXE)' )
        elif self.project.getTargetType() == 'dll':
            self.writeln( '\t$(_VC_MANIFEST_EMBED_DLL)' )

            
    def CleanTarget(self, targets = ""):
        # Run the base class to create the clean lines.
        umake_makefile.makefile_generator.CleanTarget( self , targets )

        # Add the manifest embedding specifics.
        toclean = self.project.getCleanItems()
        if targets or toclean:
            if self.project.getTargetType() == 'exe' or \
               self.project.getTargetType() == 'dll':
                   self.writeln( '\t$(_VC_MANIFEST_CLEAN)' )

umake_win_makefile._pGenerator = umake_win_vc7_makefile.VC7ProjectGenerator
umake_win_makefile._config     = VC8Config
umake_win_makefile._project    = VC8Project
umake_win_makefile._workspace  = VC8Workspace        

def make_makefile(platform, project):
    """Redefines the module call umake_makefile.make_makefile() to use our 
    overridden makefile_generator class if the embed_manifest option is 
    thrown."""
    # Right now, this is the only differnce between VC8 and standard generator.
    # This conditional will have to move inside the VC8... subclass if we add
    # additional functionality to the class.
    if project.BuildOption( "embed_manifest" ):
        umake_makefile._makefileGenerator = VC8MakefileGenerator
    # Else we will use the default mapping found in umake_makefile.
        
    return umake_win_makefile.make_makefile(platform,project)


